/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.core;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

/**
 * ImageUtil provides utilities for image transformation
 * 
 * @author Samppa Saarela
 * @version $Id: ImageUtil.java 5487 2009-04-24 08:49:46Z tiwe $
 */
public class ImageUtil {

    /**
     * @param ow
     * @param oh
     * @param nw
     * @param nh
     * @param enlarge
     * @return
     */
    public static AffineTransform getResizeTransform(double ow, double oh,
            double nw, double nh, boolean enlarge) {
        double ratio = getAspectRatio(ow, oh, nw, nh, enlarge);
        return new AffineTransform(ratio, 0.0d, 0.0d, ratio, 0.0d, 0.0d);
    }

    /**
     * @param ow
     * @param oh
     * @param nw
     * @param nh
     * @param enlarge
     * @return
     */
    public static float getAspectRatio(double ow, double oh, double nw,
            double nh, boolean enlarge) {
        float ratio = 1.0f;
        if (nw > 0 && nh > 0) {
            double dw = ow - nw;
            double dh = oh - nw;
            if (dw > dh) {
                ratio = (float) (nw / ow);
            } else {
                ratio = (float) (nh / oh);
            }
        }
        if (!enlarge && ratio > 1.0) {
            ratio = 1.0f;
        }
        return ratio;
    }

    /**
     * @param originalImage
     * @param maxWidth
     * @param maxHeight
     * @param enlarge
     * @return
     */
    public static BufferedImage resize(BufferedImage originalImage,
            double maxWidth, double maxHeight, boolean enlarge) {
        Raster original = originalImage.getRaster();
        double ow = original.getWidth();
        double oh = original.getHeight();

        // shrink
        if (ow > maxWidth || oh > maxHeight) {
            double ratio = getAspectRatio(ow, oh, maxWidth, maxHeight, enlarge);
            int nw = (int) (ow * ratio);
            int nh = (int) (oh * ratio);
            return toBufferedImage(originalImage.getScaledInstance(nw, nh,
                    Image.SCALE_SMOOTH), nw, nh);
        }
        // enlarge
        else if (enlarge && (ow < maxWidth && oh < maxHeight)) {
            AffineTransformOp filter = new AffineTransformOp(ImageUtil
                    .getResizeTransform(ow, oh, maxWidth, maxHeight, true),
                    AffineTransformOp.TYPE_BILINEAR);

            WritableRaster scaledRaster = filter
                    .createCompatibleDestRaster(original);
            filter.filter(original, scaledRaster);

            return new BufferedImage(originalImage.getColorModel(),
                    scaledRaster, false, null);
        }
        // original image is of requested size
        else {
            return originalImage;
        }
    }

    /**
     * @param image
     * @param width
     * @param height
     * @return
     */
    public static BufferedImage toBufferedImage(Image image, int width,
            int height) {
        if (image instanceof BufferedImage) {
            return (BufferedImage) image;
        } else {
            BufferedImage result = new BufferedImage(width, height,
                    BufferedImage.TYPE_INT_RGB);
            Graphics2D g = result.createGraphics();
            g.drawImage(image, 0, 0, null);
            g.dispose();
            return result;
        }
    }
}
