/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.core.collection;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * ComparableList provides a comparable list adapter implementation
 * 
 * @author Samppa Saarela
 * @version $Id: ComparableList.java 5487 2009-04-24 08:49:46Z tiwe $
 */
public class ComparableList<T> extends ListAdapter<T> implements List<T>,
        Comparable<ComparableList<T>> {

    public static <T> ComparableList<T> mergedView(ComparableList<T> head,
            T tail) {
        return new ComparableList<T>(ListUtil.mergedView(head, tail));
    }

    private Comparator<? super T> comparator = null;

    private boolean nullBeforeValue = true;

    public ComparableList() {
        super(new LinkedList<T>());
    }

    public ComparableList(Comparator<T> comparator) {
        super(new LinkedList<T>());
        this.comparator = comparator;
    }

    public ComparableList(Comparator<T> comparator, int size) {
        super(new ArrayList<T>(size));
        this.comparator = comparator;
    }

    public ComparableList(Comparator<T> comparator, List<T> source) {
        super(source);
        this.comparator = comparator;
    }

    public ComparableList(Comparator<T> comparator, T[] source) {
        super(Arrays.asList(source));
        this.comparator = comparator;
    }

    public ComparableList(int size) {
        super(new ArrayList<T>(size));
    }

    public ComparableList(List<T> source) {
        super(source);
    }

    public ComparableList(T[] source) {
        super(Arrays.asList(source));
    }

    /**
     * @see java.lang.Comparable#compareTo(T)
     */
    public int compareTo(ComparableList<T> o) {
        if (o == null)
            return nullBeforeValue ? 1 : -1;
        int rv = 0;
        Iterator<T> i1 = iterator(), i2 = o.iterator();
        for (; rv == 0 && i1.hasNext() && i2.hasNext();) {
            rv = ComparableUtil.nullSafeCompare(i1.next(), i2.next(),
                    comparator, nullBeforeValue);
        }
        if (i1.hasNext()) {
            rv = 1;
        } else if (i2.hasNext()) {
            rv = -1;
        }
        return rv;
    }

    /**
     * @param nullBeforeValue
     */
    public void setNullBeforeValue(boolean nullBeforeValue) {
        this.nullBeforeValue = nullBeforeValue;
    }

    /**
     * @param another
     * @return
     */
    public boolean startsWith(ComparableList<T> another) {
        Iterator<T> i1 = iterator(), i2 = another.iterator();
        while (i2.hasNext()) {
            if (!i1.hasNext() || !i2.next().equals(i1.next())) {
                return false;
            }
        }
        return true;
    }

    /**
     * @see java.util.List#subList(int, int)
     */
    public List<T> subList(int fromIndex, int toIndex) {
        return new ComparableList<T>(getSource().subList(fromIndex, toIndex));
    }
    
    /**
     * @return
     */
    public boolean isNullBeforeValue() {
        return nullBeforeValue;
    }

}
