/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.core.collection;

import java.util.Comparator;

/**
 * ComparableUtil provides
 * 
 * @author Samppa Saarela
 * @version $Id: ComparableUtil.java 5487 2009-04-24 08:49:46Z tiwe $
 */
public class ComparableUtil {

    /**
     * @param <T>
     * @param c1
     * @param c2
     * @return
     */
    public static <T extends Comparable<T>> int nullSafeCompare(T c1, T c2) {
        return nullSafeCompare(c1, c2, true);
    }

    /**
     * @param <T>
     * @param c1
     * @param c2
     * @param nullBeforeValue
     * @return
     */
    public static <T extends Comparable<T>> int nullSafeCompare(T c1, T c2,
            boolean nullBeforeValue) {
        return nullSafeCompare(c1, c2, null, nullBeforeValue);
    }

    /**
     * @param <T>
     * @param c1
     * @param c2
     * @param comparator
     * @return
     */
    public static <T> int nullSafeCompare(T c1, T c2, Comparator<T> comparator) {
        return nullSafeCompare(c1, c2, null, true);
    }

    /**
     * @param <T>
     * @param c1
     * @param c2
     * @param comparator
     * @param nullBeforeValue
     * @return
     */
    @SuppressWarnings("unchecked")
    public static <T> int nullSafeCompare(T c1, T c2, Comparator comparator,
            boolean nullBeforeValue) {
        if (c1 == null) {
            if (c2 == null) {
                return 0;
            } else {
                return nullBeforeValue ? -1 : 1;
            }
        } else if (c2 == null) {
            return nullBeforeValue ? 1 : -1;
        } else {
            return comparator == null ? ((Comparable) c1).compareTo(c2)
                    : comparator.compare(c1, c2);
        }
    }

}
