/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */ 
package com.mysema.commons.core.collection;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * ContextMap provides
 * 
 * @author Timo
 * 
 */
public class ContextMap<K,V> implements Map<K,V> {

    private Map<K,V> head;

    private Map<K,V> tail;

    public ContextMap(Map<K,V> head, Map<K,V> tail) {
        this.head = (head == null ? new HashMap<K,V>() : head);
        if (tail != null) {
            this.tail = Collections.unmodifiableMap(tail);
        }
    }

    public int size() {
        return head.size() + (tail == null ? 0 : tail.size());
    }

    public boolean isEmpty() {
        return head.isEmpty() && tail == null || tail.isEmpty();
    }

    public boolean containsKey(Object key) {
        return head.containsKey(key) || tail != null && tail.containsKey(key);
    }

    public boolean containsValue(Object value) {
        return head.containsValue(value) || tail != null && tail.containsValue(value);
    }

    public V get(Object key) {
        V value = head.get(key);
        if (value == null && tail != null) {
            value = tail.get(key);
        }
        return value;
    }

    public V put(K key, V value) {
        return head.put(key, value);
    }

    public V remove(Object key) {
        return head.remove(key);
    }

    public void putAll(Map<? extends K,? extends V> map) {
        head.putAll(map);
    }

    public void clear() {
        head.clear();
    }

    public Set<K> keySet() {
        Set<K> keySet = head.keySet();
        if (tail != null) {
            keySet.addAll(tail.keySet());
        }
        return keySet;
    }

    public Collection<V> values() {
        Collection<V> values = head.values();
        if (tail != null) {
            values.addAll(tail.values());
        }
        return values;
    }

    public Set<Map.Entry<K, V>> entrySet() {
        Set<Map.Entry<K, V>> entrySet = head.entrySet();
        if (tail != null) {
            entrySet.addAll(tail.entrySet());
        }
        return entrySet;
    }

    public String toString() {
        return head.toString() + (tail == null ? "" : " -> " + tail.toString());
    }
}
