/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.core.collection;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * ListAdapter provides a general list adapter implementation
 * 
 * @author Timo Westk&auml;mper
 * @version $Id$
 */
public class ListAdapter<T> implements List<T> {
    
    private List<T> source;
    
    public ListAdapter(List<T> source){
        this.source = source;
    }
    
    /**
     * @see java.util.List#add(int, E)
     */
    public void add(int index, T element) {
        source.add(index, element);
    }
    
    /**
     * @see java.util.List#add(E)
     */
    public boolean add(T o) {
        return source.add(o);
    }

    /**
     * @see java.util.List#addAll(java.util.Collection)
     */
    public boolean addAll(Collection<? extends T> c) {
        return source.addAll(c);
    }

    /**
     * @see java.util.List#addAll(int, java.util.Collection)
     */
    public boolean addAll(int index, Collection<? extends T> c) {
        return source.addAll(index, c);
    }

    /**
     * @see java.util.List#clear()
     */
    public void clear() {
        source.clear();
    }

    /**
     * @see java.util.List#contains(java.lang.Object)
     */
    public boolean contains(Object o) {
        return source.contains(o);
    }
    

    /**
     * @see java.util.List#containsAll(java.util.Collection)
     */
    public boolean containsAll(Collection<?> c) {
        return source.containsAll(c);
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        return source.equals(o);
    }

    /**
     * @see java.util.List#get(int)
     */
    public T get(int index) {
        return source.get(index);
    }

    protected List<T> getSource(){
        return source;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode(){
        return source.hashCode();
    }



    /**
     * @see java.util.List#indexOf(java.lang.Object)
     */
    public int indexOf(Object o) {
        return source.indexOf(o);
    }

    /**
     * @see java.util.List#isEmpty()
     */
    public boolean isEmpty() {
        return source.isEmpty();
    }

    /**
     * @see java.util.List#iterator()
     */
    public Iterator<T> iterator() {
        return source.iterator();
    }

    /**
     * @see java.util.List#lastIndexOf(java.lang.Object)
     */
    public int lastIndexOf(Object o) {
        return source.lastIndexOf(o);
    }

    /**
     * @see java.util.List#listIterator()
     */
    public ListIterator<T> listIterator() {
        return source.listIterator();
    }

    /**
     * @see java.util.List#listIterator(int)
     */
    public ListIterator<T> listIterator(int index) {
        return source.listIterator(index);
    }

    /**
     * @see java.util.List#remove(int)
     */
    public T remove(int index) {
        return source.remove(index);
    }

    /**
     * @see java.util.List#remove(java.lang.Object)
     */
    public boolean remove(Object o) {
        return source.remove(o);
    }

    /**
     * @see java.util.List#removeAll(java.util.Collection)
     */
    public boolean removeAll(Collection<?> c) {
        return source.removeAll(c);
    }

    /**
     * @see java.util.List#retainAll(java.util.Collection)
     */
    public boolean retainAll(Collection<?> c) {
        return source.removeAll(c);
    }
    

    /**
     * @see java.util.List#set(int, E)
     */
    public T set(int index, T element) {
        return source.set(index, element);
    }

    /**
     * @see java.util.List#size()
     */
    public int size() {
        return source.size();
    }

    /* (non-Javadoc)
     * @see java.util.List#subList(int, int)
     */
    public List<T> subList(int fromIndex, int toIndex) {
        return source.subList(fromIndex, toIndex);
    }
    
    public String toString(){
        return source.toString();
    }
    
    /**
     * @see java.util.List#toArray()
     */
    public Object[] toArray() {
        return source.toArray();
    }
    
    /**
     * @see java.util.List#toArray(T[])
     */
    @SuppressWarnings("hiding")
    public <T> T[] toArray(T[] a) {
        return source.<T> toArray(a);
    }
}
