/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.core.collection;

import java.util.AbstractList;
import java.util.Collections;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * ListUtil provides
 * 
 * @author Samppa Saarela
 * @version $Id: ListUtil.java 5487 2009-04-24 08:49:46Z tiwe $
 */
public final class ListUtil {

    /**
     * @param list
     * @param minSize
     */
    public static void ensureSize(List<?> list, int minSize) {
        while (list.size() < minSize) {
            list.add(null);
        }
    }

    /**
     * @param <T>
     * @param head
     * @param tail
     * @return
     */
    public static <T> List<T> mergedView(List<T> head, Object tail) {
        return mergedView(head, Collections.singletonList(tail));
    }

    /**
     * @param <T>
     * @param head
     * @param tail
     * @return
     */
    public static <T> List<T> mergedView(final List<T> head, final List<T> tail) {
        if (head == null) {
            throw new IllegalArgumentException("head was null");
        } else if (tail == null) {
            throw new IllegalArgumentException("tail was null");
        }
        return new AbstractList<T>() {

            @Override
            public T get(int index) {
                T o;
                if (index < head.size()) {
                    o = head.get(index);
                } else {
                    index = index - head.size();
                    if (index < tail.size()) {
                        o = tail.get(index);
                    } else {
                        throw new NoSuchElementException();
                    }
                }
                return o;
            }

            @Override
            public int size() {
                return head.size() + tail.size();
            }

        };
    }

}
